/*
 * Copyright (c) 2017 Oracle and/or its affiliates. All rights reserved. This
 * code is released under a tri EPL/GPL/LGPL license. You can use it,
 * redistribute it and/or modify it under the terms of the:
 *
 * Eclipse Public License version 1.0, or
 * GNU General Public License version 2, or
 * GNU Lesser General Public License version 2.1.
 */
package org.truffleruby.core.inlined;

import org.truffleruby.RubyContext;
import org.truffleruby.core.numeric.FloatNodes;
import org.truffleruby.core.numeric.IntegerNodes.ModNode;
import org.truffleruby.core.numeric.IntegerNodesFactory.ModNodeFactory;
import org.truffleruby.language.dispatch.RubyCallNodeParameters;

import com.oracle.truffle.api.CompilerDirectives;
import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.frame.VirtualFrame;

public abstract class InlinedModNode extends BinaryInlinedOperationNode {

    @Child ModNode fixnumMod;

    public InlinedModNode(RubyContext context, RubyCallNodeParameters callNodeParameters) {
        super(callNodeParameters,
                context.getCoreMethods().integerModAssumption,
                context.getCoreMethods().floatModAssumption);
    }

    // We need to avoid the % 0 case as it would give a wrong Ruby backtrace.

    @Specialization(guards = "b != 0", assumptions = "assumptions")
    Object intMod(int a, int b) {
        return getModNode().executeMod(a, b);
    }

    @Specialization(guards = "b != 0", assumptions = "assumptions")
    Object longMod(long a, long b) {
        return getModNode().executeMod(a, b);
    }

    protected static final double ZERO = 0.0;

    @Specialization(guards = "b != ZERO", assumptions = "assumptions")
    Object floatMod(double a, double b,
            @Cached("create()") FloatNodes.ModNode modNode) {
        return modNode.executeMod(a, b);
    }

    @Specialization
    Object fallback(VirtualFrame frame, Object a, Object b) {
        return rewriteAndCall(frame, a, b);
    }

    private ModNode getModNode() {
        if (fixnumMod == null) {
            CompilerDirectives.transferToInterpreterAndInvalidate();
            fixnumMod = insert(ModNodeFactory.create(null));
        }
        return fixnumMod;
    }

}
