/*
 * Copyright (c) 2015, 2017 Oracle and/or its affiliates. All rights reserved. This
 * code is released under a tri EPL/GPL/LGPL license. You can use it,
 * redistribute it and/or modify it under the terms of the:
 *
 * Eclipse Public License version 1.0, or
 * GNU General Public License version 2, or
 * GNU Lesser General Public License version 2.1.
 */
package org.truffleruby.core.support;

import com.oracle.truffle.api.CompilerDirectives.TruffleBoundary;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.source.SourceSection;
import org.truffleruby.builtins.Primitive;
import org.truffleruby.builtins.PrimitiveArrayArgumentsNode;

/**
 * Catch-all class for primitives that are invoked but haven't yet been defined.  Its only purpose is to
 * allow Truffle to parse primitive calls without failing during the translation phase.  If any code ever
 * executes nodes here, things will break and you must implement the primitive in its respective parent.
 */
public abstract class UndefinedPrimitiveNodes {

    @Primitive(name = "undefined")
    public static abstract class UndefinedPrimitiveNode extends PrimitiveArrayArgumentsNode {

        @TruffleBoundary
        @Specialization
        public Object undefined(Object args) {
            final SourceSection sourceSection = getEncapsulatingSourceSection();
            throw new UnsupportedOperationException(
                    "Undefined primitive: \"" + sourceSection.toString().trim() + '"');
        }

    }

}
