# Copyright (c) 2014, 2017 Oracle and/or its affiliates. All rights reserved. This
# code is released under a tri EPL/GPL/LGPL license. You can use it,
# redistribute it and/or modify it under the terms of the:
#
# Eclipse Public License version 1.0, or
# GNU General Public License version 2, or
# GNU Lesser General Public License version 2.1.

# Copyright (c) 2007-2015, Evan Phoenix and contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of Rubinius nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

module Truffle::Internal

  module Unsafe
    def self.set_class(obj, cls)
      Truffle.primitive :vm_set_class

      if obj.kind_of? ImmediateValue
        raise TypeError, 'Can not change the class of an immediate'
      end

      raise ArgumentError, "Class #{cls} is not compatible with #{obj.inspect}"
    end
  end
  
  def self.get_data(path, offset)
    file = File.open(path)
    file.seek(offset)

    # flock() does not work on Solaris if the File is not open in write mode
    unless Truffle::Platform.solaris?
      if Truffle::POSIX.respond_to? :flock
        # I think if the file can't be locked then we just silently ignore
        if file.flock(File::LOCK_EX | File::LOCK_NB)
          Truffle::KernelOperations.at_exit true do
            file.flock(File::LOCK_UN)
          end
        end
      end
    end

    file
  end
  
  def self.load_arguments_from_array_kw_helper(array, kwrest_name, binding)
    array = array.dup

    last_arg = array.pop

    if last_arg.respond_to?(:to_hash)
      kwargs = last_arg.to_hash

      if kwargs.nil?
        array.push last_arg
        return array
      end

      raise TypeError, "can't convert #{last_arg.class} to Hash (#{last_arg.class}#to_hash gives #{kwargs.class})" unless kwargs.is_a?(Hash)

      return array + [kwargs] unless kwargs.keys.any? { |k| k.is_a? Symbol }

      kwargs.select! do |key, value|
        symbol = key.is_a? Symbol
        array.push({key => value}) unless symbol
        symbol
      end
    else
      kwargs = {}
    end

    binding.local_variable_set(kwrest_name, kwargs) if kwrest_name
    array
  end

  def self.add_rejected_kwargs_to_rest(rest, kwargs)
    return if kwargs.nil?

    rejected = kwargs.select do |key, _value|
      not key.is_a?(Symbol)
    end

    unless rejected.empty?
      rest.push rejected
    end
  end
  
  def self.when_splat(cases, expression)
    cases.any? do |c|
      c === expression
    end
  end
end
